@@ -545,6 +545,8 @@ static char* get_legacy_param_info(const char* key_name, long paramId)
         if (strcmp(key_name, "name")==0)      return "Surface net thermal radiation, clear sky";
         if (strcmp(key_name, "cfVarName")==0) return "strc";
         if (strcmp(key_name, "cfName")==0)    return "surface_net_downward_longwave_flux_assuming_clear_sky";
+    } else if (paramId == 228051 || paramId == 228053 || paramId == 228057 || paramId == 228058 || paramId == 228059 || paramId == 228060) {
+        if (strcmp(key_name, "cfName")==0)    return "unknown";
     }
     return NULL;
 }
@@ -567,7 +569,7 @@ static const char* get_ECMWF_local_parameter(grib_accessor* a, grib_handle* h)
         char* pLocalParam = NULL;
         /* Must be one of: 'name', 'shortName', 'units', 'cfName' etc */
         grib_accessor* a2 = NULL;
-        long pid_guess = guess_paramId(h);
+        const long pid_guess = guess_paramId(h);
         if (pid_guess == -1) return NULL;
 
         /* TODO: Need to revisit */
