@@ -19,6 +19,7 @@
    IMPLEMENTS = init
    IMPLEMENTS = unpack_double
    IMPLEMENTS = pack_double
+   IMPLEMENTS = unpack_double_element
    IMPLEMENTS = value_count
    MEMBERS=const char*   number_of_values
    MEMBERS=const char*   reference_value
@@ -51,6 +52,7 @@ static int unpack_double(grib_accessor*, double* val,size_t *len);
 static int value_count(grib_accessor*,long*);
 static void init(grib_accessor*,const long, grib_arguments* );
 static void init_class(grib_accessor_class*);
+static int unpack_double_element(grib_accessor*,size_t i, double* val);
 
 typedef struct grib_accessor_data_ccsds_packing {
     grib_accessor          att;
@@ -112,7 +114,7 @@ static grib_accessor_class _grib_accessor_class_data_ccsds_packing = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
-    0,     /* unpack only ith value          */
+    &unpack_double_element,     /* unpack only ith value          */
     0,     /* unpack a subarray         */
     0,              		/* clear          */
     0,               		/* clone accessor          */
@@ -149,7 +151,6 @@ static void init_class(grib_accessor_class* c)
 	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
-	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->unpack_double_subarray	=	(*(c->super))->unpack_double_subarray;
 	c->clear	=	(*(c->super))->clear;
 	c->make_clone	=	(*(c->super))->make_clone;
@@ -303,7 +304,6 @@ static int unpack_double(grib_accessor* a, double* val, size_t *len)
 cleanup:
     grib_context_buffer_free(a->context,decoded);
     return err;
-
 }
 
 static int pack_double(grib_accessor* a, const double* val, size_t *len)
@@ -520,21 +520,45 @@ cleanup:
 
     return err;
 }
-#else
 
-static int unpack_double(grib_accessor* a, double* val, size_t *len)
+static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
 {
-    grib_context_log(a->context, GRIB_LOG_ERROR,
-            "grib_accessor_data_ccsds_packing: CCSDS support not enabled. "
-            "Please rebuild with -DENABLE_AEC=ON");
-    return GRIB_FUNCTIONALITY_NOT_ENABLED;
+    /* The index idx relates to codedValues NOT values! */
+    size_t size = 0;
+    double* values = NULL;
+    int err=grib_get_size(grib_handle_of_accessor(a),"codedValues",&size);
+    if (err) return err;
+    if (idx > size) return GRIB_INVALID_NEAREST;
+
+    values=(double*)grib_context_malloc_clear(a->context,size*sizeof(double));
+    err=grib_get_double_array(grib_handle_of_accessor(a),"codedValues",values,&size);
+    if (err) return err;
+    *val=values[idx];
+    grib_context_free(a->context,values);
+    return err;
 }
 
-static int pack_double(grib_accessor* a, const double* val, size_t *len)
+#else
+
+static void print_error_msg(grib_context* c)
 {
-    grib_context_log(a->context, GRIB_LOG_ERROR,
+    grib_context_log(c, GRIB_LOG_ERROR,
             "grib_accessor_data_ccsds_packing: CCSDS support not enabled. "
             "Please rebuild with -DENABLE_AEC=ON");
+}
+static int unpack_double(grib_accessor* a, double* val, size_t *len)
+{
+    print_error_msg(a->context);
+    return GRIB_FUNCTIONALITY_NOT_ENABLED;
+}
+static int pack_double(grib_accessor* a, const double* val, size_t *len)
+{
+    print_error_msg(a->context);
+    return GRIB_FUNCTIONALITY_NOT_ENABLED;
+}
+static int unpack_double_element(grib_accessor* a, size_t idx, double* val)
+{
+    print_error_msg(a->context);
     return GRIB_FUNCTIONALITY_NOT_ENABLED;
 }
 
