@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -21,7 +21,6 @@
 #include <string.h>
 #include <sys/types.h>
 
-/* Microsoft Windows Visual Studio support */
 #include "eccodes_windef.h"
 
 #ifndef ECCODES_ON_WINDOWS
@@ -189,7 +188,7 @@ typedef struct grib_multi_handle    grib_multi_handle;
 */
 typedef struct grib_context   grib_context;
 
-/*! Grib iterator, structure supporting a geographic iteration of values on a GRIB message.
+/*! Grib geoiterator, structure supporting a geographic iteration of values on a GRIB message.
     \ingroup grib_iterator
 */
 typedef struct grib_iterator  grib_iterator;
@@ -293,7 +292,7 @@ grib_index* grib_index_read(grib_context* c,const char* filename,int *err);
  * @param size        : number of distinct values of the key in the index
  * @return            0 if OK, integer value on error
  */
-int grib_index_get_size(grib_index* index,const char* key,size_t* size);
+int grib_index_get_size(const grib_index* index,const char* key,size_t* size);
 
 /**
  *  Get the distinct values of the key in argument contained in the index. The key must belong to the index. This function is used when the type of the key was explicitly defined as long or when the native type of the key is long.
@@ -305,7 +304,7 @@ int grib_index_get_size(grib_index* index,const char* key,size_t* size);
  * @param size        : size of the values array
  * @return            0 if OK, integer value on error
  */
-int grib_index_get_long(grib_index* index,const char* key,
+int grib_index_get_long(const grib_index* index,const char* key,
                         long* values,size_t *size);
 
 /**
@@ -318,7 +317,7 @@ int grib_index_get_long(grib_index* index,const char* key,
  * @param size        : size of the values array
  * @return            0 if OK, integer value on error
  */
-int grib_index_get_double(grib_index* index,const char* key,
+int grib_index_get_double(const grib_index* index,const char* key,
                           double* values,size_t *size);
 
 /**
@@ -331,7 +330,7 @@ int grib_index_get_double(grib_index* index,const char* key,
  * @param size        : size of the values array
  * @return            0 if OK, integer value on error
  */
-int grib_index_get_string(grib_index* index,const char* key,
+int grib_index_get_string(const grib_index* index,const char* key,
                           char** values,size_t *size);
 
 
@@ -374,7 +373,7 @@ int grib_index_select_string(grib_index* index,const char* key,char* value);
  * When no more handles are available from the index a NULL pointer is returned and the err variable is set to GRIB_END_OF_INDEX.
  *
  * @param index       : an index created from a file.
- * @param err         :  0 if OK, integer value on error. GRIB_END_OF_INDEX when no more handles are contained in the index.
+ * @param err         : 0 if OK, integer value on error. GRIB_END_OF_INDEX when no more handles are contained in the index.
  * @return            grib handle.
  */
 grib_handle* grib_handle_new_from_index(grib_index* index,int *err);
@@ -434,7 +433,7 @@ grib_handle* grib_handle_new_from_file(grib_context* c, FILE* f, int* error);
 * @param mode        : mode
 * @return            0 if OK, integer value on error
 */
-int grib_write_message(grib_handle* h,const char* file,const char* mode);
+int grib_write_message(const grib_handle* h,const char* file,const char* mode);
 
 typedef struct grib_string_list grib_string_list;
 struct grib_string_list {
@@ -503,7 +502,7 @@ grib_handle* grib_handle_new_from_samples (grib_context* c, const char* sample_n
 * @param h           : The handle to be cloned
 * @return            the new handle, NULL if the message is invalid or a problem is encountered
 */
-grib_handle* grib_handle_clone             (grib_handle* h)                 ;
+grib_handle* grib_handle_clone(const grib_handle* h)                 ;
 
 /**
 *  Frees a handle, also frees the message if it is not a user message
@@ -511,7 +510,7 @@ grib_handle* grib_handle_clone             (grib_handle* h)                 ;
 * @param h           : The handle to be deleted
 * @return            0 if OK, integer value on error
 */
-int   grib_handle_delete   (grib_handle* h);
+int grib_handle_delete(grib_handle* h);
 
 /**
  *  Create an empty multi field handle.
@@ -566,7 +565,7 @@ int grib_multi_handle_write(grib_multi_handle* mh,FILE* f);
 * @param message_length : On exit, the message size in number of bytes
 * @return            0 if OK, integer value on error
 */
-int grib_get_message(grib_handle* h ,const void** message, size_t *message_length);
+int grib_get_message(const grib_handle* h ,const void** message, size_t *message_length);
 
 
 /**
@@ -578,21 +577,21 @@ int grib_get_message(grib_handle* h ,const void** message, size_t *message_lengt
 *                         On exit, the actual message length in number of bytes
 * @return            0 if OK, integer value on error
 */
-int grib_get_message_copy(grib_handle* h ,  void* message,size_t *message_length);
+int grib_get_message_copy(const grib_handle* h, void* message,size_t *message_length);
 /*! @} */
 
 /*! \defgroup iterators Iterating on latitude/longitude/values */
 /*! @{ */
 
 /*!
-* \brief Create a new iterator from a handle, using current geometry and values.
+* \brief Create a new geoiterator from a handle, using current geometry and values.
 *
-* \param h           : the handle from which the iterator will be created
+* \param h           : the handle from which the geoiterator will be created
 * \param flags       : flags for future use.
 * \param error       : error code
-* \return            the new iterator, NULL if no iterator can be created
+* \return            the new geoiterator, NULL if no geoiterator can be created
 */
-grib_iterator*      grib_iterator_new      (grib_handle*   h, unsigned long flags,int* error);
+grib_iterator* grib_iterator_new(const grib_handle* h, unsigned long flags, int* error);
 
 /**
 * Get latitude/longitude and data values.
@@ -605,62 +604,62 @@ grib_iterator*      grib_iterator_new      (grib_handle*   h, unsigned long flag
 * @param values      : returned array of data values
 * @return            0 if OK, integer value on error
 */
-int grib_get_data(grib_handle *h, double *lats, double *lons, double *values);
+int grib_get_data(const grib_handle *h, double *lats, double *lons, double *values);
 
 /**
-* Get the next value from an iterator.
+* Get the next value from a geoiterator.
 *
-* @param i           : the iterator
+* @param i           : the geoiterator
 * @param lat         : on output latitude in degree
 * @param lon         : on output longitude in degree
 * @param value       : on output value of the point
 * @return            positive value if successful, 0 if no more data are available
 */
-int                 grib_iterator_next     (grib_iterator *i, double* lat,double* lon,double* value);
+int grib_iterator_next(grib_iterator *i, double* lat,double* lon,double* value);
 
 /**
-* Get the previous value from an iterator.
+* Get the previous value from a geoiterator.
 *
-* @param i           : the iterator
+* @param i           : the geoiterator
 * @param lat         : on output latitude in degree
 * @param lon         : on output longitude in degree
 * @param value       : on output value of the point*
 * @return            positive value if successful, 0 if no more data are available
 */
-int                 grib_iterator_previous (grib_iterator *i, double* lat,double* lon,double* value);
+int grib_iterator_previous(grib_iterator *i, double* lat,double* lon,double* value);
 
 /**
-* Test procedure for values in an iterator.
+* Test procedure for values in a geoiterator.
 *
-* @param i           : the iterator
-* @return            boolean, 1 if the iterator still nave next values, 0 otherwise
+* @param i           : the geoiterator
+* @return            boolean, 1 if the geoiterator still nave next values, 0 otherwise
 */
-int                 grib_iterator_has_next (grib_iterator *i);
+int grib_iterator_has_next(grib_iterator *i);
 
 /**
-* Test procedure for values in an iterator.
+* Test procedure for values in a geoiterator.
 *
-* @param i           : the iterator
+* @param i           : the geoiterator
 * @return            0 if OK, integer value on error
 */
-int                 grib_iterator_reset    (grib_iterator *i);
+int grib_iterator_reset(grib_iterator *i);
 
 /**
-*  Frees an iterator from memory
+*  Frees a geoiterator from memory
 *
-* @param i           : the iterator
+* @param i           : the geoiterator
 * @return            0 if OK, integer value on error
 */
-int                 grib_iterator_delete   (grib_iterator *i);
+int grib_iterator_delete(grib_iterator *i);
 
 /*!
-* \brief Create a new nearest from a handle, using current geometry .
+* \brief Create a new nearest neighbour object from a handle, using current geometry.
 *
-* \param h           : the handle from which the iterator will be created
+* \param h           : the handle from which the nearest object will be created
 * \param error       : error code
 * \return            the new nearest, NULL if no nearest can be created
 */
-grib_nearest*      grib_nearest_new      (grib_handle*   h, int* error);
+grib_nearest* grib_nearest_new(const grib_handle* h, int* error);
 
 /**
 * Find the 4 nearest points of a latitude longitude point.
@@ -683,9 +682,9 @@ grib_nearest*      grib_nearest_new      (grib_handle*   h, int* error);
 * @param len         : size of the arrays
 * @return            0 if OK, integer value on error
 */
-int grib_nearest_find(grib_nearest *nearest,grib_handle* h,double inlat,double inlon,
-    unsigned long flags,double* outlats,double* outlons,
-    double* values,double* distances,int* indexes,size_t *len);
+int grib_nearest_find(grib_nearest* nearest, const grib_handle* h, double inlat, double inlon,
+                      unsigned long flags, double* outlats, double* outlons,
+                      double* values, double* distances, int* indexes, size_t* len);
 
 /**
 *  Frees an nearest from memory
@@ -693,7 +692,7 @@ int grib_nearest_find(grib_nearest *nearest,grib_handle* h,double inlat,double i
 * @param nearest           : the nearest
 * @return            0 if OK, integer value on error
 */
-int                 grib_nearest_delete   (grib_nearest *nearest);
+int grib_nearest_delete(grib_nearest *nearest);
 
 /**
 * Find the nearest point of a set of points whose latitudes and longitudes
@@ -719,10 +718,10 @@ int                 grib_nearest_delete   (grib_nearest *nearest);
 * @param indexes     : returned array of indexes of the nearest points
 * @return            0 if OK, integer value on error
 */
-int grib_nearest_find_multiple(grib_handle* h,int is_lsm,
-    double* inlats,double* inlons,long npoints,
-    double* outlats,double* outlons,
-    double* values,double* distances, int* indexes);
+int grib_nearest_find_multiple(const grib_handle* h, int is_lsm,
+    const double* inlats, const double* inlons, long npoints,
+    double* outlats, double* outlons,
+    double* values, double* distances, int* indexes);
 
 /* @} */
 
@@ -737,7 +736,7 @@ int grib_nearest_find_multiple(grib_handle* h,int is_lsm,
 * @param offset      : the address of a size_t where the offset will be set
 * @return            0 if OK, integer value on error
 */
-int                  grib_get_offset(grib_handle* h, const char* key, size_t* offset);
+int grib_get_offset(const grib_handle* h, const char* key, size_t* offset);
 
 /**
 *  Get the number of coded value from a key, if several keys of the same name are present, the total sum is returned
@@ -747,7 +746,7 @@ int                  grib_get_offset(grib_handle* h, const char* key, size_t* of
 * @param size        : the address of a size_t where the size will be set
 * @return            0 if OK, integer value on error
 */
-int         grib_get_size(grib_handle* h, const char* key,size_t *size);
+int grib_get_size(const grib_handle* h, const char* key,size_t *size);
 
 /**
 *  Get the length of the string representation of the key, if several keys of the same name are present, the maximum length is returned
@@ -757,7 +756,7 @@ int         grib_get_size(grib_handle* h, const char* key,size_t *size);
 * @param length        : the address of a size_t where the length will be set
 * @return            0 if OK, integer value on error
 */
-int         grib_get_length(grib_handle* h, const char* key,size_t *length);
+int grib_get_length(const grib_handle* h, const char* key,size_t *length);
 
 /**
 *  Get a long value from a key, if several keys of the same name are present, the last one is returned
@@ -768,7 +767,7 @@ int         grib_get_length(grib_handle* h, const char* key,size_t *length);
 * @param value       : the address of a long where the data will be retrieved
 * @return            0 if OK, integer value on error
 */
-int          grib_get_long         (grib_handle* h, const char* key, long*   value);
+int grib_get_long(const grib_handle* h, const char* key, long* value);
 
 /**
 *  Get a double value from a key, if several keys of the same name are present, the last one is returned
@@ -779,7 +778,7 @@ int          grib_get_long         (grib_handle* h, const char* key, long*   val
 * @param value       : the address of a double where the data will be retrieved
 * @return            0 if OK, integer value on error
 */
-int grib_get_double       (grib_handle* h, const char* key, double* value);
+int grib_get_double(const grib_handle* h, const char* key, double* value);
 
 /**
 *  Get as double the i-th element of the "key" array
@@ -790,7 +789,7 @@ int grib_get_double       (grib_handle* h, const char* key, double* value);
 * @param value       : the address of a double where the data will be retrieved
 * @return            0 if OK, integer value on error
 */
-int grib_get_double_element(grib_handle* h, const char* key, int i, double* value);
+int grib_get_double_element(const grib_handle* h, const char* key, int i, double* value);
 
 /**
 *  Get as double array the elements of the "key" array whose indexes are listed in the input array i
@@ -802,7 +801,7 @@ int grib_get_double_element(grib_handle* h, const char* key, int i, double* valu
 * @param value       : the double array for the data values
 * @return            0 if OK, integer value on error
 */
-int grib_get_double_elements(grib_handle* h, const char* key, int* i, long size, double* value);
+int grib_get_double_elements(const grib_handle* h, const char* key, int* i, long size, double* value);
 
 /**
 *  Get a string value from a key, if several keys of the same name are present, the last one is returned
@@ -814,7 +813,7 @@ int grib_get_double_elements(grib_handle* h, const char* key, int* i, long size,
 * @param length      : the address of a size_t that contains allocated length of the string on input, and that contains the actual length of the string on output
 * @return            0 if OK, integer value on error
 */
-int grib_get_string(grib_handle* h, const char* key, char* mesg, size_t *length);
+int grib_get_string(const grib_handle* h, const char* key, char* mesg, size_t *length);
 
 /**
 *  Get string array values from a key. If several keys of the same name are present, the last one is returned
@@ -826,7 +825,7 @@ int grib_get_string(grib_handle* h, const char* key, char* mesg, size_t *length)
 * @param length  : the address of a size_t that contains allocated length of the array on input, and that contains the actual length of the array on output
 * @return        0 if OK, integer value on error
 */
-int grib_get_string_array(grib_handle* h, const char* key, char** vals, size_t *length);
+int grib_get_string_array(const grib_handle* h, const char* key, char** vals, size_t *length);
 
 /**
 *  Get raw bytes values from a key. If several keys of the same name are present, the last one is returned
@@ -838,7 +837,7 @@ int grib_get_string_array(grib_handle* h, const char* key, char** vals, size_t *
 * @param length      : the address of a size_t that contains allocated length of the byte array on input, and that contains the actual length of the byte array on output
 * @return            0 if OK, integer value on error
 */
-int grib_get_bytes        (grib_handle* h, const char* key, unsigned char*  bytes, size_t *length);
+int grib_get_bytes(const grib_handle* h, const char* key, unsigned char*  bytes, size_t *length);
 /**
 *  Get double array values from a key. If several keys of the same name are present, the last one is returned
 * @see  grib_set_double_array
@@ -849,7 +848,7 @@ int grib_get_bytes        (grib_handle* h, const char* key, unsigned char*  byte
 * @param length      : the address of a size_t that contains allocated length of the double array on input, and that contains the actual length of the double array on output
 * @return            0 if OK, integer value on error
 */
-int grib_get_double_array (grib_handle* h, const char* key, double* vals, size_t *length);
+int grib_get_double_array(const grib_handle* h, const char* key, double* vals, size_t *length);
 
 /**
 *  Get long array values from a key. If several keys of the same name are present, the last one is returned
@@ -861,7 +860,7 @@ int grib_get_double_array (grib_handle* h, const char* key, double* vals, size_t
 * @param length      : the address of a size_t that contains allocated length of the long array on input, and that contains the actual length of the long array on output
 * @return            0 if OK, integer value on error
 */
-int grib_get_long_array(grib_handle* h, const char* key, long* vals, size_t *length);
+int grib_get_long_array(const grib_handle* h, const char* key, long* vals, size_t *length);
 
 
 /*   setting      data         */
@@ -885,7 +884,7 @@ int grib_copy_namespace(grib_handle* dest, const char* name, grib_handle* src);
 * @param val         : a long where the data will be read
 * @return            0 if OK, integer value on error
 */
-int grib_set_long         (grib_handle* h, const char*  key , long val);
+int grib_set_long(grib_handle* h, const char* key, long val);
 
 /**
 *  Set a double value from a key. If several keys of the same name are present, the last one is set
@@ -896,7 +895,7 @@ int grib_set_long         (grib_handle* h, const char*  key , long val);
 * @param val       : a double where the data will be read
 * @return            0 if OK, integer value on error
 */
-int grib_set_double       (grib_handle* h, const char*  key , double   val);
+int grib_set_double(grib_handle* h, const char* key, double val);
 
 /**
 *  Set a string value from a key. If several keys of the same name are present, the last one is set
@@ -908,7 +907,7 @@ int grib_set_double       (grib_handle* h, const char*  key , double   val);
 * @param length      : the address of a size_t that contains the length of the string on input, and that contains the actual packed length of the string on output
 * @return            0 if OK, integer value on error
 */
-int grib_set_string       (grib_handle* h, const char*  key , const char* mesg, size_t *length);
+int grib_set_string(grib_handle* h, const char* key, const char* mesg, size_t *length);
 
 /**
 *  Set a bytes array from a key. If several keys of the same name are present, the last one is set
@@ -920,7 +919,7 @@ int grib_set_string       (grib_handle* h, const char*  key , const char* mesg,
 * @param length      : the address of a size_t that contains the length of the byte array on input, and that contains the actual packed length of the byte array  on output
 * @return            0 if OK, integer value on error
 */
-int grib_set_bytes        (grib_handle* h, const char*  key, const unsigned char* bytes, size_t *length);
+int grib_set_bytes(grib_handle* h, const char* key, const unsigned char* bytes, size_t *length);
 
 /**
 *  Set a double array from a key. If several keys of the same name are present, the last one is set
@@ -932,7 +931,7 @@ int grib_set_bytes        (grib_handle* h, const char*  key, const unsigned char
 * @param length      : a size_t that contains the length of the byte array on input
 * @return            0 if OK, integer value on error
 */
-int grib_set_double_array (grib_handle* h, const char*  key , const double*        vals   , size_t length);
+int grib_set_double_array(grib_handle* h, const char*  key , const double*        vals   , size_t length);
 
 /**
 * Same as grib_set_double_array but allows setting of READ-ONLY keys like codedValues.
@@ -951,7 +950,7 @@ int grib_set_force_double_array(grib_handle* h, const char* key, const double* v
 * @param length      : a size_t that contains the length of the long array on input
 * @return            0 if OK, integer value on error
 */
-int grib_set_long_array   (grib_handle* h, const char*  key , const long*          vals   , size_t length);
+int grib_set_long_array(grib_handle* h, const char*  key , const long* vals, size_t length);
 
 /**
 *  Set a string array from a key. If several keys of the same name are present, the last one is set
@@ -976,7 +975,7 @@ int grib_set_string_array(grib_handle* h, const char *key, const char **vals, si
 * @param option_flags : all the GRIB_DUMP_FLAG_x flags can be used
 * @param arg          : used to provide a format to output data (experimental)
 */
-void   grib_dump_content(grib_handle* h,FILE* out,const char* mode, unsigned long option_flags,void* arg);
+void grib_dump_content(const grib_handle* h,FILE* out,const char* mode, unsigned long option_flags,void* arg);
 
 /**
 *  Print all keys from the parsed definition files available in a context
@@ -984,7 +983,7 @@ void   grib_dump_content(grib_handle* h,FILE* out,const char* mode, unsigned lon
 * @param f           : the File used to print the keys on
 * @param c           : the context that contains the cached definition files to be printed
 */
-void     grib_dump_action_tree(grib_context* c,  FILE* f) ;
+void grib_dump_action_tree(grib_context* c,  FILE* f) ;
 
 /*! \defgroup context The context object
  The context is a long life configuration object of the grib_api.
@@ -999,7 +998,7 @@ void     grib_dump_action_tree(grib_context* c,  FILE* f) ;
 * @param data        : pointer to the data to be freed
 * must match @see grib_malloc_proc
 */
-typedef void  (*grib_free_proc)     (const grib_context* c, void* data);
+typedef void  (*grib_free_proc)(const grib_context* c, void* data);
 
 /**
 * Grib malloc procedure, format of a procedure referenced in the context that is used to allocate memory
@@ -1008,7 +1007,7 @@ typedef void  (*grib_free_proc)     (const grib_context* c, void* data);
 * @return              a pointer to the allocated memory, NULL if no memory can be allocated
 * must match @see grib_free_proc
 */
-typedef void* (*grib_malloc_proc)   (const grib_context* c, size_t length);
+typedef void* (*grib_malloc_proc)(const grib_context* c, size_t length);
 
 /**
 * Grib realloc procedure, format of a procedure referenced in the context that is used to reallocate memory
@@ -1017,7 +1016,7 @@ typedef void* (*grib_malloc_proc)   (const grib_context* c, size_t length);
 * @param length        : length to be allocated in number of bytes
 * @return              a pointer to the allocated memory
 */
-typedef void* (*grib_realloc_proc)   (const grib_context* c, void* data, size_t length);
+typedef void* (*grib_realloc_proc)(const grib_context* c, void* data, size_t length);
 
 /**
 * Grib loc proc, format of a procedure referenced in the context that is used to log internal messages
@@ -1026,7 +1025,7 @@ typedef void* (*grib_realloc_proc)   (const grib_context* c, void* data, size_t
 * @param level         : the log level, as defined in log modes
 * @param mesg          : the message to be logged
 */
-typedef void  (*grib_log_proc)      (const grib_context* c, int level, const char* mesg);
+typedef void  (*grib_log_proc)(const grib_context* c, int level, const char* mesg);
 
 /**
 * Grib print proc, format of a procedure referenced in the context that is used to print external messages
@@ -1035,7 +1034,7 @@ typedef void  (*grib_log_proc)      (const grib_context* c, int level, const cha
 * @param descriptor    : the structure to be printed on, must match the implementation
 * @param mesg          : the message to be printed
 */
-typedef void  (*grib_print_proc)    (const grib_context* c, void* descriptor, const char* mesg);
+typedef void  (*grib_print_proc)(const grib_context* c, void* descriptor, const char* mesg);
 
 
 /**
@@ -1047,7 +1046,7 @@ typedef void  (*grib_print_proc)    (const grib_context* c, void* descriptor, co
 * @param stream       : the stream
 * @return              size read
 */
-typedef size_t  (*grib_data_read_proc) (const grib_context* c,void *ptr, size_t size, void *stream);
+typedef size_t (*grib_data_read_proc)(const grib_context* c,void *ptr, size_t size, void *stream);
 
 /**
 * Grib data read write, format of a procedure referenced in the context that is used to write to a stream from a resource
@@ -1058,7 +1057,7 @@ typedef size_t  (*grib_data_read_proc) (const grib_context* c,void *ptr, size_t
 * @param stream       : the stream
 * @return              size written
 */
-typedef size_t  (*grib_data_write_proc)(const grib_context* c,const void *ptr, size_t size,  void *stream);
+typedef size_t (*grib_data_write_proc)(const grib_context* c,const void *ptr, size_t size,  void *stream);
 
 /**
 * Grib data tell, format of a procedure referenced in the context that is used to tell the current position in a stream
@@ -1067,7 +1066,7 @@ typedef size_t  (*grib_data_write_proc)(const grib_context* c,const void *ptr, s
 * @param stream       : the stream
 * @return              the position in the stream
 */
-typedef off_t    (*grib_data_tell_proc) (const grib_context* c, void *stream);
+typedef off_t  (*grib_data_tell_proc) (const grib_context* c, void *stream);
 
 /**
 * Grib data seek, format of a procedure referenced in the context that is used to seek the current position in a stream
@@ -1080,7 +1079,7 @@ typedef off_t    (*grib_data_tell_proc) (const grib_context* c, void *stream);
 * @param stream       : the stream
 * @return            0 if OK, integer value on error
 */
-typedef off_t    (*grib_data_seek_proc) (const grib_context* c, off_t offset, int whence, void *stream);
+typedef off_t  (*grib_data_seek_proc)(const grib_context* c, off_t offset, int whence, void *stream);
 
 /**
 * Grib data eof, format of a procedure referenced in the context that is used to test end of file
@@ -1089,21 +1088,21 @@ typedef off_t    (*grib_data_seek_proc) (const grib_context* c, off_t offset, in
 * @param stream       : the stream
 * @return              the position in the stream
 */
-typedef int    (*grib_data_eof_proc) (const grib_context* c, void *stream);
+typedef int (*grib_data_eof_proc) (const grib_context* c, void *stream);
 
 /**
 *  Get the static default context
 *
 * @return            the default context, NULL it the context is not available
 */
-grib_context*    grib_context_get_default(void);
+grib_context* grib_context_get_default(void);
 
 /**
 *  Frees the cached definition files of the context
 *
 * @param c           : the context to be deleted
 */
-void             grib_context_delete(grib_context* c);
+void grib_context_delete(grib_context* c);
 
 /**
 *  Set the GTS header mode on.
@@ -1134,7 +1133,7 @@ void grib_gribex_mode_on(grib_context* c);
 *
 * @param c           : the context
 */
-int grib_get_gribex_mode ( grib_context* c);
+int grib_get_gribex_mode(grib_context* c);
 
 /**
 *  Set the GRIBEX mode off.
@@ -1179,7 +1178,7 @@ void grib_context_set_memory_proc(grib_context* c, grib_malloc_proc griballoc,
 * @param gribfree    : the memory freeing procedure to be set @see grib_free_proc
 */
 void  grib_context_set_persistent_memory_proc(grib_context* c, grib_malloc_proc griballoc,
-  grib_free_proc gribfree);
+                    grib_free_proc gribfree);
 
 /**
 *  Sets memory procedures of the context for large buffers
@@ -1188,9 +1187,9 @@ void  grib_context_set_persistent_memory_proc(grib_context* c, grib_malloc_proc
 * @param griballoc   : the memory allocation procedure to be set @see grib_malloc_proc
 * @param gribfree    : the memory freeing procedure to be set @see grib_free_proc
 */
-void  grib_context_set_buffer_memory_proc(grib_context* c, grib_malloc_proc griballoc,
-       grib_free_proc gribfree,
-       grib_realloc_proc gribrealloc);
+void grib_context_set_buffer_memory_proc(grib_context* c, grib_malloc_proc griballoc,
+                                         grib_free_proc gribfree,
+                                         grib_realloc_proc gribrealloc);
 
 /**
 *  Sets the context printing procedure used for user interaction
@@ -1198,7 +1197,7 @@ void  grib_context_set_buffer_memory_proc(grib_context* c, grib_malloc_proc grib
 * @param c            : the context to be modified
 * @param printp       : the printing procedure to be set @see grib_print_proc
 */
-void   grib_context_set_print_proc(grib_context* c, grib_print_proc printp);
+void grib_context_set_print_proc(grib_context* c, grib_print_proc printp);
 
 /**
 *  Sets the context logging procedure used for system (warning, errors, infos ...) messages
@@ -1206,7 +1205,7 @@ void   grib_context_set_print_proc(grib_context* c, grib_print_proc printp);
 * @param c            : the context to be modified
 * @param logp         : the logging procedure to be set @see grib_log_proc
 */
-void    grib_context_set_logging_proc(grib_context* c, grib_log_proc logp);
+void grib_context_set_logging_proc(grib_context* c, grib_log_proc logp);
 
 /**
 *  Turn on support for multiple fields in single grib messages
@@ -1257,8 +1256,6 @@ const char* grib_get_package_name(void);
 
 /**
 *  Prints the API version
-*
-*
 */
 void grib_print_api_version(FILE* out);
 
@@ -1291,8 +1288,8 @@ int codes_bufr_keys_iterator_next(bufr_keys_iterator* kiter);
 *  @param kiter         : valid grib_keys_iterator
 *  @return              key name
 */
-const char* grib_keys_iterator_get_name(grib_keys_iterator *kiter);
-char* codes_bufr_keys_iterator_get_name(bufr_keys_iterator* kiter);
+const char* grib_keys_iterator_get_name(const grib_keys_iterator *kiter);
+char* codes_bufr_keys_iterator_get_name(const bufr_keys_iterator* kiter);
 
 /*! Delete the iterator.
 *  @param kiter         : valid grib_keys_iterator
@@ -1310,10 +1307,10 @@ int codes_bufr_keys_iterator_rewind(bufr_keys_iterator* kiter);
 
 int grib_keys_iterator_set_flags(grib_keys_iterator *kiter,unsigned long flags);
 
-int grib_keys_iterator_get_long(grib_keys_iterator *kiter, long *v, size_t *len);
-int grib_keys_iterator_get_double(grib_keys_iterator *kiter, double *v, size_t *len);
-int grib_keys_iterator_get_string(grib_keys_iterator *kiter, char *v, size_t *len);
-int grib_keys_iterator_get_bytes(grib_keys_iterator *kiter, unsigned char *v, size_t *len);
+int grib_keys_iterator_get_long(const grib_keys_iterator *kiter, long *v, size_t *len);
+int grib_keys_iterator_get_double(const grib_keys_iterator *kiter, double *v, size_t *len);
+int grib_keys_iterator_get_string(const grib_keys_iterator *kiter, char *v, size_t *len);
+int grib_keys_iterator_get_bytes(const grib_keys_iterator *kiter, unsigned char *v, size_t *len);
 int codes_copy_key(grib_handle* h1,grib_handle* h2,const char* key,int type);
 
 /* @} */
@@ -1329,7 +1326,7 @@ void grib_update_sections_lengths(grib_handle* h);
 const char* grib_get_error_message(int code);
 const char* grib_get_type_name(int type);
 
-int grib_get_native_type(grib_handle* h, const char* name,int* type);
+int grib_get_native_type(const grib_handle* h, const char* name,int* type);
 
 void grib_check(const char* call,const char*  file,int line,int e,const char* msg);
 #define GRIB_CHECK(a,msg) grib_check(#a,__FILE__,__LINE__,a,msg)
@@ -1339,8 +1336,8 @@ void grib_check(const char* call,const char*  file,int line,int e,const char* ms
 int grib_set_values(grib_handle* h,grib_values*  grib_values , size_t arg_count);
 grib_handle* grib_handle_new_from_partial_message_copy(grib_context* c, const void* data, size_t size);
 grib_handle* grib_handle_new_from_partial_message(grib_context* c, const void* data, size_t buflen);
-int grib_is_missing(grib_handle* h, const char* key, int* err);
-int grib_is_defined(grib_handle* h, const char* key);
+int grib_is_missing(const grib_handle* h, const char* key, int* err);
+int grib_is_defined(const grib_handle* h, const char* key);
 int grib_set_missing(grib_handle* h, const char* key);
 /* The truncation is the Gaussian number (or order) */
 int grib_get_gaussian_latitudes(long truncation,double* latitudes);
@@ -1351,6 +1348,7 @@ long grib_julian_to_date(long jdate);
 long grib_date_to_julian(long ddate);
 
 void grib_get_reduced_row(long pl,double lon_first,double lon_last,long* npoints,long* ilon_first, long* ilon_last);
+void grib_get_reduced_row_p(long pl, double lon_first, double lon_last, long *npoints, double *olon_first, double *olon_last);
 
 /* read products */
 int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
@@ -1358,13 +1356,17 @@ int wmo_read_grib_from_file(FILE* f,void* buffer,size_t* len);
 int wmo_read_bufr_from_file(FILE* f,void* buffer,size_t* len);
 int wmo_read_gts_from_file(FILE* f,void* buffer,size_t* len);
 int wmo_read_any_from_stream(void *stream_data, long (*stream_proc )(void *, void *buffer, long len ), void *buffer, size_t *len);
+
+/* These functions allocate memory for the result so the user is responsible for freeing it */
 void* wmo_read_any_from_stream_malloc(void* stream_data,long (*stream_proc)(void*,void* buffer,long len) ,size_t *size, int* err);
 void *wmo_read_any_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err);
 void *wmo_read_gts_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err);
 void *wmo_read_bufr_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err);
 void *wmo_read_grib_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err);
-int grib_get_message_offset ( grib_handle* h,off_t* offset);
-int grib_get_message_size ( grib_handle* h,size_t* size);
+
+int grib_read_any_from_file(grib_context *ctx, FILE *f, void *buffer, size_t *len);
+int grib_get_message_offset(const grib_handle* h,off_t* offset);
+int grib_get_message_size(const grib_handle* h,size_t* size);
 
 struct grib_points {
     grib_context* context;
@@ -1384,10 +1386,6 @@ int grib_points_get_values(grib_handle* h, grib_points* points, double* val);
 
 
 /* --------------------------------------- */
-
-
-
-
 #define GRIB_UTIL_GRID_SPEC_REGULAR_LL 1
 #define GRIB_UTIL_GRID_SPEC_ROTATED_LL 2
 
@@ -1397,9 +1395,10 @@ int grib_points_get_values(grib_handle* h, grib_points* points, double* val);
 
 #define GRIB_UTIL_GRID_SPEC_SH         6
 #define GRIB_UTIL_GRID_SPEC_REDUCED_LL 7
-#define GRIB_UTIL_GRID_SPEC_POLAR_STEREOGRAPHIC 8
-
-#define GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG 9
+#define GRIB_UTIL_GRID_SPEC_POLAR_STEREOGRAPHIC          8
+#define GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG           9
+#define GRIB_UTIL_GRID_SPEC_LAMBERT_AZIMUTHAL_EQUAL_AREA 10
+#define GRIB_UTIL_GRID_SPEC_LAMBERT_CONFORMAL            11
 
 
 typedef struct grib_util_grid_spec {
@@ -1498,14 +1497,15 @@ typedef struct grib_util_grid_spec2 {
 
 } grib_util_grid_spec2;
 
-#define GRIB_UTIL_PACKING_TYPE_SAME_AS_INPUT      0
-#define GRIB_UTIL_PACKING_TYPE_SPECTRAL_COMPLEX 1
-#define GRIB_UTIL_PACKING_TYPE_SPECTRAL_SIMPLE    2
-#define GRIB_UTIL_PACKING_TYPE_JPEG    3
-#define GRIB_UTIL_PACKING_TYPE_GRID_COMPLEX    4
-#define GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE    5
-#define GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE_MATRIX    6
-#define GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER    7
+#define GRIB_UTIL_PACKING_TYPE_SAME_AS_INPUT       0
+#define GRIB_UTIL_PACKING_TYPE_SPECTRAL_COMPLEX    1
+#define GRIB_UTIL_PACKING_TYPE_SPECTRAL_SIMPLE     2
+#define GRIB_UTIL_PACKING_TYPE_JPEG                3
+#define GRIB_UTIL_PACKING_TYPE_GRID_COMPLEX        4
+#define GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE         5
+#define GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE_MATRIX  6
+#define GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER   7
+#define GRIB_UTIL_PACKING_TYPE_CCSDS               8
 
 #define GRIB_UTIL_PACKING_SAME_AS_INPUT    	0
 #define GRIB_UTIL_PACKING_USE_PROVIDED  	1
@@ -1558,8 +1558,84 @@ grib_handle *grib_util_set_spec2(grib_handle *h,
     size_t data_values_count,
     int *err);
 
-/* --------------------------------------- */
+int parse_keyval_string(const char *grib_tool, char *arg, int values_required, int default_type, grib_values values[], int *count);
+grib_handle *grib_new_from_file(grib_context *c, FILE *f, int headers_only, int *error);
 
+/* EXPERIMENTAL */
+typedef struct codes_bufr_header {
+    unsigned long message_offset;
+    unsigned long message_size;
+
+    /* Section 0 keys */
+    long edition;
+
+    /* Section 1 keys */
+    long masterTableNumber;
+    long bufrHeaderSubCentre;
+    long bufrHeaderCentre;
+    long updateSequenceNumber;
+    long dataCategory;
+    long dataSubCategory;
+    long masterTablesVersionNumber;
+    long localTablesVersionNumber;
+
+    long typicalYear;
+    long typicalMonth;
+    long typicalDay;
+    long typicalHour;
+    long typicalMinute;
+    long typicalSecond;
+    long typicalDate; /* computed key */
+    long typicalTime; /* computed key */
+
+    long internationalDataSubCategory; /*BUFR4-specific*/
+
+    long localSectionPresent;
+    long ecmwfLocalSectionPresent;
+
+    /* ECMWF local section keys */
+    long rdbType;
+    long oldSubtype;
+    long rdbSubtype;
+    char ident[9];
+    long localYear;
+    long localMonth;
+    long localDay;
+    long localHour;
+    long localMinute;
+    long localSecond;
+
+    long rdbtimeDay;
+    long rdbtimeHour;
+    long rdbtimeMinute;
+    long rdbtimeSecond;
+
+    long rectimeDay;
+    long rectimeHour;
+    long rectimeMinute;
+    long rectimeSecond;
+
+    long   isSatellite;
+    double localLongitude1;
+    double localLatitude1;
+    double localLongitude2;
+    double localLatitude2;
+    double localLatitude;
+    double localLongitude;
+    long   localNumberOfObservations;
+    long   satelliteID;
+    long   qualityControl;
+    long   newSubtype;
+    long   daLoop;
+
+    /* Section 3 keys */
+    unsigned long numberOfSubsets;
+    long observedData;
+    long compressedData;
+
+} codes_bufr_header;
+
+/* --------------------------------------- */
 
 typedef void (*codes_assertion_failed_proc)(const char* message);
 void codes_set_codes_assertion_failed_proc(codes_assertion_failed_proc proc);
@@ -1709,5 +1785,7 @@ Error codes returned by the grib_api functions.
 #define GRIB_OUT_OF_RANGE		-65
 /** Size of bitmap is incorrect */
 #define GRIB_WRONG_BITMAP_SIZE		-66
+/** Functionality not enabled */
+#define GRIB_FUNCTIONALITY_NOT_ENABLED		-67
 /*! @}*/
 #endif
