@@ -64,6 +64,7 @@
    #include <dirent.h>
    #include <unistd.h>
    #include <inttypes.h>
+   #define ecc_snprintf snprintf
 #else
    #include <direct.h>
    #include <io.h>
@@ -99,6 +100,8 @@
    #    define strdup(str)         _strdup(str)
    #  endif
 
+   #define ecc_snprintf _snprintf
+
 #endif
 
 
