@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -10,6 +10,7 @@
 
 #include "grib_api_internal.h"
 #include <errno.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #ifndef ECCODES_ON_WINDOWS
 #include <unistd.h>
@@ -246,11 +247,10 @@ void grib_print_api_version(FILE* out)
             ECCODES_MAJOR_VERSION,
             ECCODES_MINOR_VERSION,
             ECCODES_REVISION_VERSION);
-    /*
-    * if (ECCODES_MAJOR_VERSION < 1) {
-    *    fprintf(out, "%s", " PRE-RELEASE");
-    * }
-    */
+
+    if (ECCODES_MAJOR_VERSION < 1) {
+        fprintf(out, "%s", " PRE-RELEASE");
+    }
 }
 
 const char* grib_get_package_name()
@@ -336,7 +336,7 @@ static grib_context default_grib_context = {
         0,                            /* ieee_packing               */
         0,                            /* bufrdc_mode                */
         0,                            /* bufr_set_to_missing_if_out_of_range */
-        0,                            /* bufr_quality_without_bitmap*/
+        0,                            /* bufr_multi_element_constant_arrays */
         0,                            /* log_stream                 */
         0,                            /* classes                    */
         0,                            /* lists                      */
@@ -355,10 +355,6 @@ grib_context* grib_context_get_default()
     GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_c);
 
-#ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
-    feenableexcept(FE_ALL_EXCEPT & ~FE_INEXACT);
-#endif
-
     if(!default_grib_context.inited)
     {
         const char* write_on_fail = NULL;
@@ -374,13 +370,17 @@ grib_context* grib_context_get_default()
         const char* keep_matrix = NULL;
         const char* bufrdc_mode = NULL;
         const char* bufr_set_to_missing_if_out_of_range = NULL;
-        const char* bufr_quality_without_bitmap = NULL;
+        const char* bufr_multi_element_constant_arrays = NULL;
         const char* file_pool_max_opened_files = NULL;
 
+#ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
+        feenableexcept(FE_ALL_EXCEPT & ~FE_INEXACT);
+#endif
+
         write_on_fail = codes_getenv("ECCODES_GRIB_WRITE_ON_FAIL");
-        bufrdc_mode = codes_getenv("ECCODES_BUFRDC_MODE_ON");
-        bufr_set_to_missing_if_out_of_range = codes_getenv("ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE");
-        bufr_quality_without_bitmap = codes_getenv("ECCODES_BUFR_QUALITY_WITHOUT_BITMAP");
+        bufrdc_mode = getenv("ECCODES_BUFRDC_MODE_ON");
+        bufr_set_to_missing_if_out_of_range = getenv("ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE");
+        bufr_multi_element_constant_arrays = getenv("ECCODES_BUFR_MULTI_ELEMENT_CONSTANT_ARRAYS");
         large_constant_fields = codes_getenv("ECCODES_GRIB_LARGE_CONSTANT_FIELDS");
         no_abort = codes_getenv("ECCODES_NO_ABORT");
         debug = codes_getenv("ECCODES_DEBUG");
@@ -391,7 +391,7 @@ grib_context* grib_context_get_default()
         no_big_group_split = codes_getenv("ECCODES_GRIB_NO_BIG_GROUP_SPLIT");
         no_spd = codes_getenv("ECCODES_GRIB_NO_SPD");
         keep_matrix = codes_getenv("ECCODES_GRIB_KEEP_MATRIX");
-        file_pool_max_opened_files = codes_getenv("ECCODES_FILE_POOL_MAX_OPENED_FILES");
+        file_pool_max_opened_files = getenv("ECCODES_FILE_POOL_MAX_OPENED_FILES");
 
         /* On UNIX, when we read from a file we get exactly what is in the file on disk.
          * But on Windows a file can be opened in binary or text mode. In binary mode the system behaves exactly as in UNIX.
@@ -477,8 +477,8 @@ grib_context* grib_context_get_default()
         default_grib_context.bufrdc_mode = bufrdc_mode ? atoi(bufrdc_mode) : 0;
         default_grib_context.bufr_set_to_missing_if_out_of_range = bufr_set_to_missing_if_out_of_range ?
                 atoi(bufr_set_to_missing_if_out_of_range) : 0;
-        default_grib_context.bufr_quality_without_bitmap = bufr_quality_without_bitmap ?
-                atoi(bufr_quality_without_bitmap) : 0;
+        default_grib_context.bufr_multi_element_constant_arrays = bufr_multi_element_constant_arrays ?
+                atoi(bufr_multi_element_constant_arrays) : 0;
         default_grib_context.file_pool_max_opened_files = file_pool_max_opened_files ?
                 atoi(file_pool_max_opened_files) : DEFAULT_FILE_POOL_MAX_OPENED_FILES;
     }
@@ -623,7 +623,9 @@ char *grib_context_full_defs_path(grib_context* c,const char* basename)
     if(*basename == '/' || *basename ==  '.') {
         return (char*)basename;
     } else {
+        GRIB_MUTEX_LOCK(&mutex_c);  /* See ECC-604 */
         fullpath=(grib_string_list*)grib_trie_get(c->def_files,basename);
+        GRIB_MUTEX_UNLOCK(&mutex_c);
         if (fullpath!=NULL) {
             return fullpath->value;
         }
@@ -743,6 +745,19 @@ void grib_context_delete( grib_context* c)
         grib_context_free_persistent(&default_grib_context,c);
 }
 
+void codes_bufr_multi_element_constant_arrays_on(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->bufr_multi_element_constant_arrays = 1;
+}
+void codes_bufr_multi_element_constant_arrays_off(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->bufr_multi_element_constant_arrays = 0;
+}
+/*int  codes_get_bufr_multi_element_constant_arrays(grib_context* c);*/
+
+
 void grib_context_set_definitions_path(grib_context* c, const char* path)
 {
     if (!c) c=grib_context_get_default();
@@ -949,6 +964,27 @@ void grib_context_print(const grib_context *c, void* descriptor,const char* fmt,
     c->print(c,descriptor,msg);
 }
 
+int grib_context_get_handle_file_count(grib_context *c)
+{
+    int r = 0;
+    if (!c) c=grib_context_get_default();
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
+    r = c->handle_file_count;
+    GRIB_MUTEX_UNLOCK(&mutex_c);
+    return r;
+}
+int grib_context_get_handle_total_count(grib_context *c)
+{
+    int r = 0;
+    if (!c) c=grib_context_get_default();
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
+    r=c->handle_total_count;
+    GRIB_MUTEX_UNLOCK(&mutex_c);
+    return r;
+}
+
 void grib_context_set_handle_file_count(grib_context *c, int new_count)
 {
     if (!c) c=grib_context_get_default();
@@ -957,7 +993,6 @@ void grib_context_set_handle_file_count(grib_context *c, int new_count)
     c->handle_file_count = new_count;
     GRIB_MUTEX_UNLOCK(&mutex_c);
 }
-
 void grib_context_set_handle_total_count(grib_context *c, int new_count)
 {
     if (!c) c=grib_context_get_default();
@@ -975,7 +1010,6 @@ void grib_context_increment_handle_file_count(grib_context *c)
     c->handle_file_count++;
     GRIB_MUTEX_UNLOCK(&mutex_c);
 }
-
 void grib_context_increment_handle_total_count(grib_context *c)
 {
     if (!c) c=grib_context_get_default();
@@ -1076,3 +1110,41 @@ void codes_assertion_failed(const char* message, const char* file, int line)
         assertion(buffer);
     }
 }
+
+int grib_get_gribex_mode(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    return c->gribex_mode_on;
+}
+void grib_gribex_mode_on(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->gribex_mode_on=1;
+}
+void grib_gribex_mode_off(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->gribex_mode_on=0;
+}
+
+void grib_gts_header_on(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->gts_header_on=1;
+}
+void grib_gts_header_off(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->gts_header_on=0;
+}
+
+void grib_multi_support_on(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->multi_support_on=1;
+}
+void grib_multi_support_off(grib_context* c)
+{
+    if ( !c ) c=grib_context_get_default();
+    c->multi_support_on=0;
+}
